import os

from cpl_core.application import StartupABC
from cpl_core.configuration import ConfigurationABC, ConsoleArgument
from cpl_core.dependency_injection import ServiceProviderABC, ServiceCollectionABC
from cpl_core.environment import ApplicationEnvironment

from py_to_uxf_core.abc.attribute_scanner_abc import AttributeScannerABC
from py_to_uxf_core.abc.class_scanner_abc import ClassScannerABC
from py_to_uxf_core.abc.file_scanner_abc import FileScannerABC
from py_to_uxf_core.abc.function_scanner_abc import FunctionScannerABC
from py_to_uxf_core.abc.implementation_scanner_abc import ImplementationScannerABC
from py_to_uxf_core.abc.python_parser_abc import PythonParserABC
from py_to_uxf_core.abc.umlet_creator_abc import UmletCreatorABC
from py_to_uxf_core.service.attribute_scanner_service import AttributeScannerService
from py_to_uxf_core.service.class_scanner_service import ClassScannerService
from py_to_uxf_core.service.file_scanner_service import FileScannerService
from py_to_uxf_core.service.function_scanner_service import FunctionScannerService
from py_to_uxf_core.service.implementation_scanner_service import ImplementationScannerService
from py_to_uxf_core.service.python_parser_service import PythonParserService
from py_to_uxf_core.service.umlet_creator_service import UmletCreatorService


class Startup(StartupABC):

    def __init__(self):
        StartupABC.__init__(self)

    def configure_configuration(self, configuration: ConfigurationABC, environment: ApplicationEnvironment) -> ConfigurationABC:
        environment.set_runtime_directory(os.path.dirname(__file__))
        if os.path.isfile('py_to_uxf/appsettings.json'):
            configuration.add_json_file('py_to_uxf/appsettings.json', optional=False, output=False)
        else:
            configuration.add_json_file('appsettings.json', optional=False, output=False)

        configuration.add_console_argument(ConsoleArgument('-', 'p', [], ' ', False, [
            ConsoleArgument('-', 'o', [], ' ', is_value_token_optional=False)
        ]))
        configuration.add_console_arguments(error=False)
        return configuration

    def configure_services(self, services: ServiceCollectionABC, environment: ApplicationEnvironment) -> ServiceProviderABC:
        services.add_transient(FileScannerABC, FileScannerService)
        services.add_transient(ClassScannerABC, ClassScannerService)
        services.add_transient(FunctionScannerABC, FunctionScannerService)
        services.add_transient(AttributeScannerABC, AttributeScannerService)
        services.add_transient(ImplementationScannerABC, ImplementationScannerService)

        services.add_singleton(PythonParserABC, PythonParserService)
        services.add_singleton(UmletCreatorABC, UmletCreatorService)

        return services.build_service_provider()
