from typing import Optional

from cpl_core.console import Console

from py_to_uxf_core.abc.attribute_scanner_abc import AttributeScannerABC
from py_to_uxf_core.model.access_modifier_enum import AccessModifierEnum
from py_to_uxf_core.model.python_class_attribute import PythonClassAttribute


class AttributeScannerService(AttributeScannerABC):

    def __init__(self):
        AttributeScannerABC.__init__(self)

    def scan_line_for_attribute(self, line: str) -> Optional[PythonClassAttribute]:
        # ATR=1
        # ATR:int=1
        # self.ATR=1
        # self._ATR=1
        # self.__ATR=1
        # self._ATR:int=1
        if '=' in line:
            line.replace(' ', '')
            name = line.split('=')[0]

            if name.startswith('self.'):
                name = name.split('self.')[1]

            if ':' in name:
                name = name.split(':')[0]

            if '.' in name:
                return None

            access_modifier = AccessModifierEnum.public
            type_str = 'any'

            # access_modifier
            if name == '__init__' or name == '__repr__' or name == '__str__':
                access_modifier = AccessModifierEnum.public
            elif name.startswith('_'):
                access_modifier = AccessModifierEnum.protected
            elif name.startswith('__'):
                access_modifier = AccessModifierEnum.private

            if ':' in line:
                type_str = line.split(':')[1]
                if '=' in type_str:
                    type_str = type_str.split('=')[0]

            return PythonClassAttribute(access_modifier, name, type_str)

        return None
